@@ -312,6 +312,10 @@ shade <- function(slope, aspect, angle=45, direction=0, normalize=FALSE, filenam
 
 
 map.pal <- function(name, n=50, ...) { 
+	n <- round(n)
+	if (n < 1) {
+		error("map.pal", "n should be > 0")	
+	}
 	f <- system.file("colors/palettes.rds", package="terra")
 	v <- readRDS(f)
 	if (missing(name)) {
@@ -324,6 +328,13 @@ map.pal <- function(name, n=50, ...) {
 		} else {
 			v
 		}
+	} else if (name == "random") {
+		if (n > 433) {
+			error("map.pal", "you cannot get more than 433 random colors")
+		}
+		s <- sample(grDevices::colors()[grep('gr(a|e)y', grDevices::colors(), invert = TRUE)], n)
+		rgb(t(grDevices::col2rgb(s))/255)
+
 	} else {
 		error("map.pal", paste(name, "is not a known palette"))
 	}
