@@ -43,14 +43,14 @@ setMethod("dots", signature(x="SpatVector"),
 #	cols <- out$cols
 #	if (is.null(cols)) cols = rep("black", n)
 
-#	g <- lapply(x@pnt$linesList(), function(i) { names(i)=c("x", "y"); i } )
+#	g <- lapply(x@ptr$linesList(), function(i) { names(i)=c("x", "y"); i } )
 
 #	g <- geom(x, df=TRUE)
 #	g <- split(g, g[,1])
 #	g <- lapply(g, function(x) split(x[,3:4], x[,2]))
 #	n <- length(g)
 
-	g <- x@pnt$linesList()
+	g <- x@ptr$linesList()
 	lty <- rep_len(lty, n)
 	lwd <- rep_len(lwd, n)
 	for (i in 1:n) {
@@ -119,7 +119,7 @@ setMethod("dots", signature(x="SpatVector"),
 #				# g[[i]][[1]] <- a
 #			}
 
-	g <- x@pnt$polygonsList()
+	g <- x@ptr$polygonsList()
 	if (is.null(out$leg$density)) {
 		for (i in seq_along(g)) {
 			for (j in seq_along(g[[i]])) {
@@ -293,16 +293,23 @@ setMethod("dots", signature(x="SpatVector"),
 	levs <- levels(fz)
 	nlevs <- length(levs)
 
-	cols <- out$cols
-	ncols <- length(cols)
-	if (nlevs < ncols) {
-		i <- trunc((ncols / nlevs) * 1:nlevs)
-		cols <- cols[i]
+#	cols <- out$cols
+#	ncols <- length(cols)
+#	if (nlevs < ncols) {
+#		i <- trunc((ncols / nlevs) * 1:nlevs)
+#		cols <- cols[i]
+#	} else {
+#		cols <- rep_len(cols, nlevs)
+#	}
+	
+	if (out$legend_type == "classes" && nlevs > length(out$cols)) {
+		# more classes than colors: cycle from beginning
+		cols_idx <- rep_len(out$cols, nlevs)
 	} else {
-		cols <- rep_len(cols, nlevs)
+		# else sample the colors
+		cols_idx <- trunc(seq(1, length(out$cols), length.out = nlevs))
 	}
-	out$cols <- cols
-	out$leg$fill <- cols
+	out$leg$fill <- out$cols <- out$cols[cols_idx]
 	out$legend_type <- "classes"
 
 	if (!is.null(out$leg$legend)) {
@@ -335,7 +342,7 @@ setMethod("dots", signature(x="SpatVector"),
 		if (!any(is.na(out$mar))) { graphics::par(mar=out$mar) }
 		plot(out$lim[1:2], out$lim[3:4], type="n", xlab="", ylab="", asp=out$asp, xaxs="i", yaxs="i", axes=FALSE, main="")
 		if (!is.null(out$background)) {
-			graphics::rect(out$lim[1], out$lim[3], out$lim[2], out$lim[4], col=out$background)
+			graphics::rect(out$lim[1], out$lim[3], out$lim[2], out$lim[4], col=out$background, border=TRUE)
 		}
 	}
 	if (isTRUE(out$blank)) return(out)
@@ -352,7 +359,9 @@ setMethod("dots", signature(x="SpatVector"),
 			out <- .vect.legend.interval(out, dig.lab=out$dig.lab)
 		}
 	} else if (out$legend_type == "depends") {
-		if (nuq < 11) {
+		if (!is.null(out$breaks)) {
+			out <- .vect.legend.interval(out, dig.lab=out$dig.lab)			
+		} else if (nuq < 11) {
 			out <- .vect.legend.classes(out)
 		} else if (!is.numeric(out$uv)) {
 			#if (nuq < 21)
@@ -424,7 +433,7 @@ setMethod("dots", signature(x="SpatVector"),
 
 
 .prep.vect.data <- function(x, y, type=NULL, cols=NULL, mar=NULL, legend=TRUE,
-	legend.only=FALSE, levels=NULL, add=FALSE, range=NULL, breaks=NULL, breakby="eqint",
+	legend.only=FALSE, levels=NULL, add=FALSE, range=NULL, fill_range=FALSE, breaks=NULL, breakby="eqint",
 	xlim=NULL, ylim=NULL, colNA=NA, alpha=NULL, axes=TRUE, buffer=TRUE, background=NULL,
 	pax=list(), plg=list(), ext=NULL, grid=FALSE, las=0, sort=TRUE, decreasing=FALSE, values=NULL,
 	box=TRUE, xlab="", ylab="", cex.lab=0.8, line.lab=1.5, yaxs="i", xaxs="i", main="", cex.main=1.2, line.main=0.5, font.main=graphics::par()$font.main, col.main = graphics::par()$col.main, 
@@ -570,6 +579,7 @@ setMethod("dots", signature(x="SpatVector"),
 		}
 		out$range_set <- FALSE
 	}
+	out$fill_range <- fill_range
 	out$v <- v
 
 	if (!is.logical(sort)) {
@@ -577,6 +587,9 @@ setMethod("dots", signature(x="SpatVector"),
 		out$legend_sort <- FALSE
 	} else {
 		out$uv <- unique(out$v)
+		if (is.factor(out$v)) {
+			out$uv <- levels(out$v)[levels(out$v) %in% out$uv]
+		}
 		out$legend_sort <- isTRUE(sort)
 		out$legend_sort_decreasing <- isTRUE(decreasing)
 	}
@@ -647,6 +660,9 @@ setMethod("plot", signature(x="SpatVector", y="character"),
 	sort=TRUE, decreasing=FALSE, plg=list(), pax=list(), nr, nc, colNA=NA, 
 	alpha=NULL, box=axes, clip=TRUE, ...) {
 
+		old.mar <- graphics::par()$mar
+		on.exit(graphics::par(mar=old.mar))
+
 		if (nrow(x) == 0) {
 			error("plot", "SpatVector has zero geometries")
 		}
@@ -737,3 +753,34 @@ setMethod("plot", signature(x="SpatVectorProxy", y="missing"),
 	}
 )
 
+
+
+setMethod("plot", signature(x="SpatVectorCollection", y="missing"),
+	function(x, y, main, mar=NULL, nc, nr, maxnl=16, ...) {
+		nl <- max(1, min(length(x), maxnl))
+
+		if (nl==1) {
+			if (missing(main)) main = ""
+			out <- plot(x[1], main=main[1], mar=mar, ...)
+			return(invisible(out))
+		}
+
+		nrnc <- .get_nrnc(nr, nc, nl)
+		old.par <- graphics::par(no.readonly = TRUE)
+		on.exit(graphics::par(old.par))
+		if (is.null(mar)) {
+			mar=c(1.5, 1, 2.5, 3)
+		}
+		graphics::par(mfrow=nrnc)
+
+		if (missing("main")) {
+			main <- names(x)
+		} else {
+			main <- rep_len(main, nl)
+		}
+		for (i in 1:nl) {
+			plot(x[i], main=main[i], mar=mar, ...)
+		}
+	}
+)
+
