@@ -3,12 +3,6 @@
 # Version 1.0
 # License GPL v3
 
-readAll <- function(x) {
-	ok <- x@ptr$readAll()
-	x <- messages(x, "readAll")
-	invisible(ok)
-}
-
 setMethod("readStart", signature(x="SpatRaster"),
 	function(x) {
 		success <- x@ptr$readStart()
@@ -57,3 +51,19 @@ setMethod("readStop", signature(x="SpatRasterDataset"),
 	}
 )
 
+setMethod("toMemory", signature(x="SpatRaster"),
+	function(x) {
+		x@ptr <- x@ptr$deepcopy()
+		x@ptr$readAll()
+		messages(x, "toMemory")
+	}
+)
+
+setMethod("toMemory", signature(x="SpatRasterDataset"),
+	function(x) {
+		x@ptr <- x@ptr$deepcopy()
+		x@ptr$readAll()
+		messages(x, "toMemory")
+	}
+)
+
